@@ -5,10 +5,9 @@
  */
 
 #include "window.h"
-#include "../qwaylandlayersurface_p.h"
 #include <layershellqt_logging.h>
-#include <private/qwaylandshellsurface_p.h>
-#include <private/qwaylandwindow_p.h>
+
+#include <QPointer>
 
 using namespace LayerShellQt;
 
@@ -27,7 +26,7 @@ public:
     Window::KeyboardInteractivity keyboardInteractivity = Window::KeyboardInteractivityExclusive;
     Window::Layer layer = Window::LayerTop;
     QMargins margins;
-    QWaylandLayerSurface *getSurface() const;
+    QPointer<QScreen> desiredOutput;
 };
 
 static QMap<QWindow *, Window *> s_map;
@@ -40,9 +39,7 @@ Window::~Window()
 void Window::setAnchors(Anchors anchors)
 {
     d->anchors = anchors;
-    if (auto surface = d->getSurface()) {
-        surface->setAnchor(anchors);
-    }
+    Q_EMIT anchorsChanged();
 }
 
 Window::Anchors Window::anchors() const
@@ -53,9 +50,7 @@ Window::Anchors Window::anchors() const
 void Window::setExclusiveZone(int32_t zone)
 {
     d->exclusionZone = zone;
-    if (auto surface = d->getSurface()) {
-        surface->setExclusiveZone(zone);
-    }
+    Q_EMIT exclusionZoneChanged();
 }
 
 int32_t Window::exclusionZone() const
@@ -66,9 +61,7 @@ int32_t Window::exclusionZone() const
 void Window::setMargins(const QMargins &margins)
 {
     d->margins = margins;
-    if (auto surface = d->getSurface()) {
-        surface->setMargins(margins);
-    }
+    Q_EMIT marginsChanged();
 }
 
 QMargins Window::margins() const
@@ -79,9 +72,7 @@ QMargins Window::margins() const
 void Window::setKeyboardInteractivity(KeyboardInteractivity interactivity)
 {
     d->keyboardInteractivity = interactivity;
-    if (auto surface = d->getSurface()) {
-        surface->setKeyboardInteractivity(interactivity);
-    }
+    Q_EMIT keyboardInteractivityChanged();
 }
 
 Window::KeyboardInteractivity Window::keyboardInteractivity() const
@@ -92,9 +83,6 @@ Window::KeyboardInteractivity Window::keyboardInteractivity() const
 void Window::setLayer(Layer layer)
 {
     d->layer = layer;
-    if (auto surface = d->getSurface()) {
-        surface->setLayer(layer);
-    }
 }
 
 void Window::setScope(const QString &scope)
@@ -113,6 +101,16 @@ Window::Layer Window::layer() const
     return d->layer;
 }
 
+QScreen *Window::desiredOutput() const
+{
+    return d->desiredOutput;
+}
+
+void Window::setDesiredOutput(QScreen *output)
+{
+    d->desiredOutput = output;
+}
+
 Window::Window(QWindow *window)
     : QObject(window)
     , d(new WindowPrivate(window))
@@ -128,21 +126,3 @@ Window *Window::get(QWindow *window)
     }
     return new Window(window);
 }
-
-QWaylandLayerSurface *WindowPrivate::getSurface() const
-{
-    if (!parentWindow) {
-        return nullptr;
-    }
-    auto ww = dynamic_cast<QtWaylandClient::QWaylandWindow *>(parentWindow->handle());
-    if (!ww) {
-        qCDebug(LAYERSHELLQT) << "window not a wayland window" << parentWindow;
-        return nullptr;
-    }
-    QWaylandLayerSurface *s = qobject_cast<QWaylandLayerSurface *>(ww->shellSurface());
-    if (!s) {
-        qCDebug(LAYERSHELLQT) << "window not using wlr-layer-shell" << parentWindow << ww->shellSurface();
-        return nullptr;
-    }
-    return s;
-}
